@@ -17,8 +17,14 @@ import time
 import numpy
 # import scipy.sparse
 # import scipy.spatial
-import nltk
 import pickle
+import nltk
+from nltk.stem import WordNetLemmatizer
+from nltk.corpus import stopwords
+import logging
+import logging.config
+import os
+from stopwords import heb_stopwords
 
 class Progress:
     def __init__(self):
@@ -32,9 +38,6 @@ class Progress:
             self.start = time.time()
             print(str(self.i)+" : "+str(elapsed))
 
-
-    
-
 def tf(word,words):
     return words.count(word)
 
@@ -207,6 +210,26 @@ def cluster_sentences(distmat):
     print(clusters)
     return clusters
 
+def get_json_data(dir):
+    data_json = []
+    statuses = {}
+    tags = {}
+    for file in os.listdir(dir):
+        if file.endswith(".json"):
+            with open(dir+'/'+file) as data_file:
+                data_json += json.load(data_file)
+    for entry in data_json:
+        if entry['model'] == "facebook_feeds.facebook_status":
+            statuses[entry['pk']] = entry['fields']['content']
+        if entry['model'] == "kikartags.taggeditem":
+            id = entry['fields']['object_id'] 
+            if id not in tags:
+                tags[id] = []
+            tags[id].append(str(entry['fields']['tag']))
+    data = [DataEntry(i,statuses[i], tags[i]) for i in statuses if i in tags]
+    return data
+     
+        
 def get_data():
     print('getting data')
     with open('db/statuses.txt') as statuses_file:
@@ -220,21 +243,39 @@ def get_data():
         if id not in tags:
             tags[id] = []
         tags[id].append(str(tag['fields']['tag']))
-    data = [(statuses[i], tags[i]) for i in statuses if i in tags]
+    data = [DataEntry(i,statuses[i], tags[i]) for i in statuses if i in tags]
     return data
-        
+
+class DataEntry:
+    def __init__(self, id = None, text = '', tags = []):
+        if id is None:
+            id = id(self)
+        self.id = id
+        self.text = text
+        self.tags = tags
+        self.feutures = None 
         
 class AutoTag:
     
+    def __init__(self):
+        logging.basicConfig(filename='log',level=logging.INFO)
+        self.wnl = WordNetLemmatizer()
 
     def clean_text(self,text):
+        text = unicode(text)
+        text = text.lower()
     #     text = re.sub('[^a-zA-Zא-ת0-9\"\\n]',' ', text)
         puncmarks = string.punctuation+'״׳'
     #     add space before punctuation   
-        text = re.sub('(\S)(['+puncmarks+']+)\s','\\1 \\2 ', text)
+        text = re.sub('(\S)(['+puncmarks+']+)','\\1 \\2 ', text)
     #     add space after punctuation   
-        text = re.sub('\s(['+puncmarks+']+)(\S)',' \\1 \\2 ', text)
+        text = re.sub('(['+puncmarks+']+)(\S)',' \\1 \\2', text)
+        
+        text = re.sub('(\w+)', self.wnl.lemmatize('\\1'),text)
+
         text = re.sub('\s+', ' ', text)
+        
+        text = ' '.join([word for word in text.split() if word.encode('utf-8') not in heb_stopwords])
 #         f = codecs.open('clean_text','w','utf-8')
 #         f.write(text)
 #         f.close()
@@ -245,50 +286,71 @@ class AutoTag:
         data_count = sorted(data_count.items(), key=lambda item: item[1],reverse = True)
         with open(name, 'w') as f:
             data_encoded = [(w.encode('utf-8'),c) for w,c in data_count]
-            json.dump(data_encoded, f, ensure_ascii=False, indent = 4, separators=[',',': '])
+#             data_encoded = [(w,c) for w,c in data_count]
+            json.dump(data_encoded, f, ensure_ascii=False, indent = 4, separators=[',',': ']) #TODO: fix
+#             pickle.dump(data_encoded, f) #TODO: fix
         return data_count
 
     def document_features(self,document):
+        document = self.clean_text(document)
         document_words = set(document.split())
-        with open('word_features','r') as word_features_file:
-            word_features = pickle.load(word_features_file)
         features = {}
-        for word in word_features:
+        for word in self.get_word_features():
             features['contains(%s)' % word] = (word in document_words)
         return features
     
+    def get_word_features(self):
+        with open('word_features','r') as word_file:
+            word_features = pickle.load(word_file)
+        return word_features
+        
+    
     def get_tags(self,data,n=5):
-        tag_count = self.count_data([tag for status,tags in data for tag in tags],'tag_count.txt')
+        tag_count = self.count_data([tag for entry in data for tag in entry.tags],'tag_count.txt')
         tags = [tag for tag,count in tag_count if count > n]
 #         tags = tags[:20]
         return tags
     
     def train(self,data):
         tags = self.get_tags(data)
-        print(len(tags))
+#         print(len(tags))
+        data = [entry for entry in data if entry.tags]
+        for entry in data:
+            entry.features = self.document_features(entry.text)
+#         print(len(data))
         classifier = {}
         for tag in tags:
-            print(tag)
-            featuresets = [(self.document_features(doc), tag in tags) for (doc,tags) in data if len(tags) > 0]
+#             print(tags.index(tag, ))
+#             featuresets = [(self.document_features(entry.text), tag in entry.tags) for entry in data if len(entry.tags) > 0]
+            featuresets = [(entry.features, tag in entry.tags) for entry in data if entry.tags]
             classifier[tag] = nltk.NaiveBayesClassifier.train(featuresets)
-#             classifier[tag].show_most_informative_features(5)
+            logging.info(str(tag)+': '+ str(classifier[tag].most_informative_features(10)))
         with open('classifier','w') as classifier_file:
             pickle.dump(classifier,classifier_file)
         return classifier
     
+    def test(self,test_set):
+        tags = self.get_tags(test_set)
+        accuracy = {}
+        with open('classifier','r') as classifier_file:
+            classifier = pickle.load(classifier_file)
+        for tag in tags:
+            featuresets = [(self.document_features(entry.text), tag in entry.tags) for entry in test_set]
+            accuracy[tag] = nltk.classify.accuracy(classifier[tag],featuresets)
+            print(tag+': '+str(accuracy[tag]))
+        return accuracy 
+    
     def test_tag(self,documents,tag,thresh):
         N = len(documents)
         with open('classifier','r') as classifier_file:
             classifier = pickle.load(classifier_file)
-#         featuresets = [(self.document_features(d), tag in c) for (d,c) in documents]
-#         print(nltk.classify.accuracy(classifier, featuresets))
         probs = []
         for i in range(N):
             if tag not in classifier:
                 continue
-            prob = classifier[tag].prob_classify(self.document_features(documents[i]))
+            prob = classifier[tag].prob_classify(self.document_features(documents[i].text))
             if prob.prob(True) > thresh:
-                probs.append((prob.prob(True),documents[i]))
+                probs.append((prob.prob(True),documents[i].text,documents[i].id))
         probs = sorted(probs,reverse=True)        
         return probs
     
@@ -299,7 +361,7 @@ class AutoTag:
         for tag in tags:
             if tag not in classifier:
                 continue
-            prob = classifier[tag].prob_classify(self.document_features(document))
+            prob = classifier[tag].prob_classify(self.document_features(document.text))
             if prob.prob(True) > thresh:
                 probs.append((prob.prob(True),tag))
         probs = sorted(probs,reverse=True)
@@ -307,32 +369,39 @@ class AutoTag:
 # most probable tag: input: status, n  output: list (top n most probable, + score)     
 
     def create_word_features(self,data):
-        word_count = self.count_data([w for s,t in data for w in s.split()],'word_count.txt')
+        word_count = self.count_data([w for entry in data for w in entry.text.split()],'word_count.txt')
         max_count = max([v for k,v in word_count])
-        word_features = [k for (k,v) in word_count if v > 10 and v < max_count/10]
+#         word_features = [k for (k,v) in word_count if v > 10 and v < max_count/10] #TODO: use something more intelligent
+        word_features = [k for k,v in word_count if v > 10 and v not in heb_stopwords]
+#         print(len(word_features))
         with open('word_features','w') as word_file:
             pickle.dump(word_features,word_file)
 
     def classify(self,data):
         print('cleaning')
-        data = [(self.clean_text(s),t) for s,t in data]
+        data = [DataEntry(entry.id,self.clean_text(entry.text),entry.tags) for entry in data]
         print('extracting word features')
         self.create_word_features(data)
-        print('classifying')
+        print('training')
         classifier = self.train(data)
-#         for status,count in test_statuses:
-#             print(self.test_doc(status, big_tags, word_features, classifier, 0.5))
 
 
 if __name__ == '__main__':
-    data = get_data()
+    data = get_json_data('db2')
     at = AutoTag()
+#     print('classifying')
 #     at.classify(data)
-    tags = at.get_tags(data)
-    documents = [s for s,t in data]
-    print(at.test_doc(documents[0], tags, 0))
-    print(tags[0])
-    print(at.test_tag(documents[:200], tags[0], 0)[0][1])
+#     print("testing")
+#     tags = at.get_tags(data)
+#     documents = [entry.text for entry in data]
+#     print('test doc')
+#     print(at.test_doc(data[0], tags, 0))
+#     print('test tag')
+#     print(tags[1])
+#     print(at.test_tag(data[:200], tags[1], 0)[0][1])
+    data = [DataEntry(entry.id,at.clean_text(entry.text),entry.tags) for entry in data]
+
+    at.create_word_features(data)
     
 #     print(tags)
 #     bag = bag_of_words(statuses,word_features)
@@ -348,4 +417,11 @@ if __name__ == '__main__':
 
 
 # interface - get tag: statuses sorted by probability to be tagged
-#             get status: tags sorted by probability to fit
\ No newline at end of file
+#             get status: tags sorted by probability to fit
+
+
+
+#TODO: use data object (with id, text and tags) instead of tuple
+#TODO: fix Dependencies
+#TODO: print most informative features to file
+#TODO: clean text in train, and test
\ No newline at end of file
